# Generated by Django 3.2 on 2021-11-04 12:45

from django.db import migrations

CREATE_VLABEL_OBJECT = "CREATE VLABEL IF NOT EXISTS object;"
DROP_VLABEL_OBJECT = "DROP VLABEL IF EXISTS object;"

CREATE_PROPERTY_INDEX = "CREATE PROPERTY INDEX IF NOT EXISTS idx_vlabel_object_uuid ON object (uuid);"
DROP_PROPERTY_INDEX = "DROP PROPERTY INDEX IF EXISTS idx_vlabel_object_uuid;"

SQL_MIGRATIONS = [
    (CREATE_VLABEL_OBJECT, DROP_VLABEL_OBJECT),
    (CREATE_PROPERTY_INDEX, DROP_PROPERTY_INDEX),
]


class Migration(migrations.Migration):

    dependencies = [
        ('urlid_graph', '0004_auto_20211101_1848'),
    ]

    operations = [
            migrations.RunSQL(sql=forward, reverse_sql=reverse, hints={"target_db": "graph"})
            for forward, reverse in SQL_MIGRATIONS
    ]
