# Generated by Django 3.2.9 on 2021-11-21 04:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('urlid_graph', '0005_auto_20211104_1245'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='objectrepository',
            table='urlid_graph_object_repository',
        ),
    ]
